#-----------------------------------------------------------------------------
#
#	A Python translation of linetool.rb.
#
#-----------------------------------------------------------------------------
# Original copyright notice:
#
# Copyright 2005-2008, Google, Inc.
#
# This software is provided as an example of using the Ruby interface
# to SketchUp.
#
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------

# This example shows the implementation of a simple line tool.	This tool
# is similar to the Pencil tool in SketchUp, but it create construction lines
# rather than edges.

from time import time
from tool import Tool

class LineTool(Tool):

	def __init__(self):
		self.ip1 = None
		self.ip2 = None
		self.xdown = 0
		self.ydown = 0

	# The activate method is called by SketchUp when the tool is first selected.
	# it is a good place to put most of your initialization
	def activate(self):
		# The Sketchup::InputPoint class is used to get 3D points from screen
		# positions.  It uses the SketchUp inferencing code.
		# In this tool, we will have two points for the endpoints of the line.
		self.ip1 = InputPoint()
		self.ip2 = InputPoint()
		self.ip = InputPoint()
		self.drawn = False

		# This sets the label for the VCB
		Sketchup.set_status_text("Length")
		
		self.reset(None)
	

	# deactivate is called when the tool is deactivated because
	# a different tool was selected
	def deactivate(self, view):
		if self.drawn:
			view.invalidate()


	# The onMouseMove method is called whenever the user moves the mouse.
	# because it is called so often, it is important to try to make it efficient.
	# In a lot of tools, your main interaction will occur in this method.
	def onMouseMove(self, flags, x, y, view):
		if self.state == 0:
			# We are getting the first	of the line.  Call the pick method
			# on the InputPoint to get a 3D position from the 2D screen position
			# that is bassed as an argument to this method.
			self.ip.pick(view, x, y)
			if self.ip != self.ip1:
				# if the point has changed from the last one we got, then
				# see if we need to display the point.	We need to display it
				# if it has a display representation or if the previous point
				# was displayed.  The invalidate method on the view is used
				# to tell the view that something has changed so that you need
				# to refresh the view.
				if self.ip.display() or self.ip1.display():
					view.invalidate()
				self.ip1.copy_ip(self.ip)
				
				# set the tooltip that should be displayed to this point
				view.tooltip = self.ip1.tooltip()
			
		else:
			# Getting the second  of the line
			# If you pass in another InputPoint on the pick method of InputPoint
			# it uses that second point to do additional inferencing such as
			# parallel to an axis.
			self.ip2.pick(view, x, y, self.ip1)
			if self.ip2.is_valid():
				view.tooltip = self.ip2.tooltip()
			view.invalidate()
			
			# Update the length displayed in the VCB
			if self.ip2.is_valid():
				length = self.ip1.position().distance(self.ip2.position())
				Sketchup.set_status_text(str(length), SB_VCB_VALUE)
			
			# Check to see if the mouse was moved far enough to create a line.
			# This is used so that you can create a line by either draggin
			# or doing click-move-click
			if abs(x - self.xdown) > 10 or abs(y - self.ydown) > 10:
				self.dragging = True

	# The onLButtonDOwn method is called when the user presses the left mouse button.
	def onLButtonDown(self, flags, x, y, view):
		# When the user clicks the first time, we switch to getting the
		# second point.	 When they click a second time we create the line
		if self.state == 0:
			self.ip1.pick(view, x, y)
			if self.ip1.is_valid():
				self.state = 1
				Sketchup.set_status_text("Select second ", SB_PROMPT)
				self.xdown = x
				self.ydown = y
		else:
			# create the line on the second click
			if self.ip2.is_valid():
				self.create_geometry(self.ip1.position(), self.ip2.position(), view)
				self.reset(view)
		# Clear any inference lock
		view.lock_inference()

	# The onLButtonUp method is called when the user releases the left mouse button.
	def onLButtonUp(self, flags, x, y, view):
		# If we are doing a drag, then create the line on the mouse up event
		if self.dragging and self.ip2.is_valid():
			self.create_geometry(self.ip1.position(), self.ip2.position(), view)
			self.reset(view)

	# onKeyDown is called when the user presses a key on the keyboard.
	# We are checking it here to see if the user pressed the shift key
	# so that we can do inference locking
	def onKeyDown(self, key, repeat, flags, view):
		if key == CONSTRAIN_MODIFIER_KEY and repeat == 1:
			self.shift_down_time = time()
			# if we already have an inference lock, then unlock it
			if view.is_inference_locked():
				# calling lock_inference with no arguments actually unlocks
				view.lock_inference()
			elif self.state == 0 and self.ip1.is_valid():
				view.lock_inference(self.ip1)
			elif self.state == 1 and self.ip2.is_valid():
				view.lock_inference(self.ip2, self.ip1)

	# onKeyUp is called when the user releases the key
	# We use this to unlock the interence
	# If the user holds down the shift key for more than 1/2 second, then we
	# unlock the inference on the release.	Otherwise, the user presses shift
	# once to lock and a second time to unlock.
	def onKeyUp(self, key, repeat, flags, view):
		if (key == CONSTRAIN_MODIFIER_KEY and
			view.is_inference_locked() and
			(time() - self.shift_down_time) > 0.5):
				view.lock_inference()

	# onUserText is called when the user enters something into the VCB
	# In this implementation, we create a line of the entered length if
	# the user types a length while selecting the second point

	def onUserText(self, text, view):
		# We only accept input when the state is 1 (i.e. getting the second point)
		# This could be enhanced to also modify the last line created if a length
		# is entered after creating a line.
		if self.state != 1:
			return
		if not self.ip2.is_valid():
			return

		# The user may type in something that we can't parse as a length
		# so we set up some exception handling to trap that
		try:
			value = to_length(text)
		except ValueError:
			UI.beep()
			print "Cannot convert %r to a Length" % text
			value = None
			Sketchup.set_status_text("", SB_VCB_VALUE)
		if not value:
			return
	
		# Compute the direction and the second point
		pt1 = self.ip1.position()
		vec = self.ip2.position() - pt1
		if vec.length() == 0.0:
			UI.beep()
			return
		
		vec.length = value
		pt2 = pt1 + vec
	
		# Create a line
		self.create_geometry(pt1, pt2, view)
		self.reset(view)

	# The draw method is called whenever the view is refreshed.	 It lets the
	# tool draw any temporary geometry that it needs to.

	def draw(self, view):
		if self.ip1.is_valid():
			if self.ip1.display():
				self.ip1.draw(view)
				self.drawn = True
			
			if self.ip2.is_valid():
				if self.ip2.display():
					self.ip2.draw(view) 
				
				# The set_color_from_line method determines what color
				# to use to draw a line based on its direction.	 For example
				# red, green or blue.
				view.set_color_from_line(self.ip1, self.ip2)
				self.draw_geometry(self.ip1.position(), self.ip2.position(), view)
				self.drawn = True

	# onCancel is called when the user hits the escape key

	def onCancel(self, flag, view):
		self.reset(view)

	# The following methods are not directly called from SketchUp.	They are
	# internal methods that are used to support the other methods in this class.
	
	# Reset the tool back to its initial state

	def reset(self, view):
		# This variable keeps track of which point we are currently getting
		self.state = 0
		
		# Display a prompt on the status bar
		Sketchup.set_status_text("Select first ", SB_PROMPT)
		
		# clear the InputPoints
		self.ip1.clear()
		self.ip2.clear()
		
		if view:
			view.tooltip = None
			if self.drawn:
				view.invalidate()
		
		self.drawn = False
		self.dragging = False


	# Create new geometry when the user has selected two points.
	
	def create_geometry(self, p1, p2, view):
		view.model().entities().add_cline(p1,p2)

	# Draw the geometry

	def draw_geometry(self, pt1, pt2, view):
		view.draw_line(pt1, pt2)


#-----------------------------------------------------------------------------
# You can use inheritance in Python to use the LineTool class as a base class
# for similar kinds of tools.  For example, this is a tool that creates
# cylinders by selecting the two  points.

class CylinderTool(LineTool):

	# The cylinder tool takes a radius when you create it
	def __init__(self, radius = 1):
		self.radius = radius
		self.numsegs = 16
		LineTool.__init__(self)

	# Now we just have to override the create_geometry method.	You could also
	# override the draw_geometry method to make it draw a closer represenation
	# to what it will really create.
	def create_geometry(self, pt1, pt2, view):
		model = view.model()
		model.start_operation("Create Cylinder")
		entities = model.entities()
		
		# First create a circle
		print "create_geometry:", pt1, pt2
		vec = pt2 - pt1
		length = vec.length()
		if length == 0.0:
			UI.beep()
			print "Cannot create a zero length cylinder"
			return
		
		circle = entities.add_circle(pt1, vec, self.radius, self.numsegs)
	
		# Now do a pushpull to create the cylinder
		face = entities.add_face(circle)
		normal = face.normal()
		if normal % vec < 0.0:
			length = -length 
		face.pushpull(length)
	
		model.commit_operation()

#-----------------------------------------------------------------------------

def line_tool():
	Sketchup.active_model().select_tool(LineTool())

def cylinder_tool():
	Sketchup.active_model().select_tool(CylinderTool())

import menus
plug = menus.get_menu("Plugins")
menu = plug.add_submenu("SuPy Examples")
menu.add_item("Construction Line Tool", line_tool)
menu.add_item("Cylinder Tool", cylinder_tool)
